(ns com.aurellem.gb.types
  (:use (com.aurellem.gb gb-driver util constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def type-code->type-name
  {0x00 :normal
   0x01 :fighting
   0x02 :flying
   0x03 :poision
   0x04 :ground
   0x05 :rock
   0x07 :bug
   0x08 :ghost
   0x14 :fire
   0x15 :water
   0x16 :grass
   0x17 :electric
   0x18 :psychic
   0x19 :ice
   0x1A :dragon    })

(def type-name->type-code
  (zipmap (vals type-code->type-name)
          (keys type-code->type-name)))

(def pokemon-1-type-start-address 0xD16F)

(defn pokemon-type-start-address [poke-num]
  (+ pokemon-1-type-start-address
     (* pokemon-record-width poke-num)))

(defn give-type
  ([^SaveState state poke-num types]
     (assert (<= 0 poke-num 5))
     (let [types*
           (if (= (count types) 1)
             [(first types) (first types)]
             types)]
       (set-memory-range
        state
        (pokemon-type-start-address poke-num)
        (map type-name->type-code types))))
  ([poke-num types]
     (give-type @current-state poke-num types)))

(defn read-type
  ([^SaveState state poke-num]
     (assert (<= 0 poke-num 5))
     (let [types-start (pokemon-type-start-address poke-num)
           [type-1 type-2]
           (subvec (vec (memory state))
                   types-start (+ 2 types-start))]
       (if (= type-1 type-2)
         [(type-code->type-name type-1)]
         (mapv type-code->type-name [type-1 type-2]))))
  ([poke-num]
     (read-type @current-state poke-num)))

(def pokemon->type
  {:abra            [:psychic]               
   :aerodactyl      [:rock :flying]          
   :alakazam        [:psychic]               
   :arbok           [:poison]                
   :arcanine        [:fire]                  
   :articuno        [:ice :flying]           
   :beedrill        [:bug :poison]           
   :bellsprout      [:grass :poison]         
   :blastoise       [:water]                 
   :bulbasaur       [:grass :poison]         
   :butterfree      [:bug :flying]           
   :caterpie        [:bug]                   
   :chansey         [:normal]                
   :charizard       [:fire :flying]          
   :charmander      [:fire]                  
   :charmeleon      [:fire]                  
   :clefable        [:normal]                
   :clefairy        [:normal]                
   :cloyster        [:water :ice]            
   :cubone          [:ground]                
   :dewgong         [:water :ice]            
   :diglett         [:ground]                
   :ditto           [:normal]                
   :dodrio          [:normal :flying]        
   :doduo           [:normal :flying]        
   :dragonair       [:dragon]                
   :dragonite       [:dragon :flying]        
   :dratini         [:dragon]                
   :drowzee         [:psychic]               
   :dugtrio         [:ground]                
   :eevee           [:normal]                
   :ekans           [:poison]                
   :electabuzz      [:electric]              
   :electrode       [:electric]              
   :exeggcute       [:grass :psychic]        
   :exeggutor       [:grass :psychic]        
   :farfetchd       [:normal :flying]        
   :fearow          [:normal :flying]        
   :flareon         [:fire]                  
   :gastly          [:ghost :poison]         
   :gengar          [:ghost :poison]         
   :geodude         [:rock :ground]          
   :gloom           [:grass :poison]         
   :golbat          [:poison :flying]        
   :goldeen         [:water]                 
   :golduck         [:water]                 
   :golem           [:rock :ground]          
   :graveler        [:rock :ground]          
   :grimer          [:poison]                
   :growlithe       [:fire]                  
   :gyarados        [:water :flying]         
   :haunter         [:ghost :poison]         
   :hitmonchan      [:fighting]              
   :hitmonlee       [:fighting]              
   :horsea          [:water]                 
   :hypno           [:psychic]               
   :ivysaur         [:grass :poison]         
   :jigglypuff      [:normal]                
   :jolteon         [:electric]              
   :jynx            [:ice :psychic]          
   :kabuto          [:rock :water]           
   :kabutops        [:rock :water]           
   :kadabra         [:psychic]               
   :kakuna          [:bug :poison]           
   :kangaskhan      [:normal]                
   :kingler         [:water]                 
   :koffing         [:poison]                
   :krabby          [:water]                 
   :lapras          [:water :ice]            
   :lickitung       [:normal]                
   :machamp         [:fighting]              
   :machoke         [:fighting]              
   :machop          [:fighting]              
   :magikarp        [:water]                 
   :magmar          [:fire]                  
   :magnemite       [:electric]              
   :magneton        [:electric]              
   :mankey          [:fighting]              
   :marowak         [:ground]                
   :meowth          [:normal]                
   :metapod         [:bug]                   
   :mew             [:psychic]               
   :mewtwo          [:psychic]               
   :moltres         [:fire :flying]          
   :mr-mime         [:psychic]               
   :muk             [:poison]                
   :nidoking        [:poison :ground]        
   :nidoqueen       [:poison :ground]        
   :nidoran-female  [:poison]                
   :nidoran-male    [:poison]                
   :nidorina        [:poison]                
   :nidorino        [:poison]                
   :ninetails       [:fire]                  
   :oddish          [:grass :poison]         
   :omanyte         [:rock :water]           
   :omastar         [:rock :water]           
   :onix            [:rock :ground]          
   :paras           [:bug :grass]            
   :parasect        [:bug :grass]            
   :persian         [:normal]                
   :pidgeot         [:normal :flying]        
   :pidgeotto       [:normal :flying]        
   :pidgey          [:normal :flying]        
   :pikachu         [:electric]              
   :pinsir          [:bug]                   
   :poliwag         [:water]                 
   :poliwhirl       [:water]                 
   :poliwrath       [:water :fighting]       
   :ponyta          [:fire]                  
   :porygon         [:normal]                
   :primeape        [:fighting]              
   :psyduck         [:water]                 
   :raichu          [:electric]              
   :rapidash        [:fire]                  
   :raticate        [:normal]                
   :rattata         [:normal]                
   :rhydon          [:ground :rock]          
   :rhyhorn         [:ground :rock]          
   :sandshrew       [:ground]                
   :sandslash       [:ground]                
   :scyther         [:bug :flying]           
   :seaking         [:water]                 
   :sedra           [:water]                 
   :seel            [:water]                 
   :shellder        [:water]                 
   :slowbro         [:water :psychic]        
   :slowpoke        [:water :psychic]        
   :snorlax         [:normal]                
   :spearow         [:normal :flying]        
   :squirtle        [:water]                 
   :starmie         [:water :psychic]        
   :staryu          [:water]                 
   :tangela         [:grass]                 
   :tauros          [:normal]                
   :tentacool       [:water :poison]         
   :tentacruel      [:water :poison]         
   :vaporeon        [:water]                 
   :venomoth        [:bug :poison]           
   :venonat         [:bug :poison]           
   :venusaur        [:grass :poison]         
   :victreebel      [:grass :poison]         
   :vileplume       [:grass :poison]         
   :voltorb         [:electric]              
   :vulpix          [:fire]                  
   :wartortle       [:water]                 
   :weedle          [:bug :poison]           
   :weepenbell      [:grass :poison]         
   :wheezing        [:poison]                
   :wigglytuff      [:normal]                
   :zapdos          [:electric :flying]      
   :zubat           [:poison :flying]})


























































































































































